@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+from abc import ABC, abstractmethod
 from functools import cached_property, reduce
 from itertools import product
 from operator import add
@@ -12,7 +13,48 @@ from sympy.physics.quantum.qubit import qubit_to_matrix
 from ewl.utils import amplitude_to_prob, cache, convert_exp_to_trig, is_unitary, number_of_qubits
 
 
-class EWL:
+class BaseEWL(ABC):
+    @cached_property
+    def number_of_players(self) -> int:
+        return len(self.players)
+
+    @cached_property
+    def J(self) -> Matrix:
+        return Matrix.hstack(*[
+            TensorProduct(*base) @ qubit_to_matrix(self.psi)
+            for base in product((self.C, self.D), repeat=number_of_qubits(self.psi))
+        ])
+
+    @cached_property
+    def J_H(self) -> Matrix:
+        return self.J.H
+
+    @cache
+    def payoff_function(self, *, player: Optional[int], simplify: bool = True):
+        if self.payoff_matrix is None:
+            raise Exception('Payoff matrix not defined')
+
+        if player is not None:
+            assert 0 <= player < self.number_of_players, 'Invalid number of player'
+
+        probs = self.probs(simplify=simplify)
+        payoff_matrix = self.payoff_matrix[player] if player is not None else reduce(add, self.payoff_matrix)
+        return sum(
+            probs[i] * payoff_matrix[idx]
+            for i, idx in enumerate(product(range(2), repeat=self.number_of_players))
+        )
+
+    @property
+    @abstractmethod
+    def params(self) -> Set[Symbol]:
+        pass
+
+    @abstractmethod
+    def probs(self, *, simplify: bool = True) -> Matrix:
+        pass
+
+
+class EWL(BaseEWL):
     def __init__(self, *, psi, C: Matrix, D: Matrix, players: Sequence[Matrix], payoff_matrix: Optional[Array] = None, check_unitary: bool = True):
         assert number_of_qubits(psi) == len(players), 'Number of qubits and players must be equal'
 
@@ -30,10 +72,6 @@ class EWL:
         self.players = players
         self.payoff_matrix = payoff_matrix
 
-    @cached_property
-    def number_of_players(self) -> int:
-        return len(self.players)
-
     @cached_property
     def params(self) -> Set[Symbol]:
         return self.psi.atoms(Symbol).union(*(
@@ -52,17 +90,6 @@ class EWL:
     def play(self, *players: Matrix) -> EWL:
         return EWL(psi=self.psi, C=self.C, D=self.D, players=players, payoff_matrix=self.payoff_matrix)
 
-    @cached_property
-    def J(self) -> Matrix:
-        return Matrix.hstack(*[
-            TensorProduct(*base) @ qubit_to_matrix(self.psi)
-            for base in product((self.C, self.D), repeat=number_of_qubits(self.psi))
-        ])
-
-    @cached_property
-    def J_H(self) -> Matrix:
-        return self.J.H
-
     @cache
     def amplitudes(self, *, simplify: bool = True) -> Matrix:
         ampl = self.J_H @ TensorProduct(*self.players) @ qubit_to_matrix(self.psi)
@@ -74,20 +101,5 @@ class EWL:
     def probs(self, *, simplify: bool = True) -> Matrix:
         return self.amplitudes(simplify=simplify).applyfunc(amplitude_to_prob)
 
-    @cache
-    def payoff_function(self, *, player: Optional[int], simplify: bool = True):
-        if self.payoff_matrix is None:
-            raise Exception('Payoff matrix not defined')
-
-        if player is not None:
-            assert 0 <= player < self.number_of_players, 'Invalid number of player'
-
-        probs = self.probs(simplify=simplify)
-        payoff_matrix = self.payoff_matrix[player] if player is not None else reduce(add, self.payoff_matrix)
-        return sum(
-            probs[i] * payoff_matrix[idx]
-            for i, idx in enumerate(product(range(2), repeat=self.number_of_players))
-        )
-
     def payoffs(self, *, simplify: bool = True):
         return tuple(self.payoff_function(player=i, simplify=simplify) for i in range(self.number_of_players))
